@@ -7,6 +7,9 @@ if version_info.major == 3 and version_info.minor < 6:
     exit(1)
 
 import argparse
+import shlex
+from os.path import abspath
+from subprocess import check_output
 from typing import Dict
 
 import uvicorn  # type: ignore
@@ -19,13 +22,16 @@ from soco_cli.api import rescan_speakers
 from soco_cli.api import run_command as sc_run
 from soco_cli.utils import version as print_version
 
-sc_app = FastAPI()
-
 # Globals
 USE_LOCAL = False
 PORT = 8000
 INFO = "SoCo-CLI HTTP API Server v" + version
 PREFIX = "SoCo-CLI: "
+MACROS: Dict[str, str] = {}
+MACRO_FILE = ""
+
+
+sc_app = FastAPI()
 
 
 def command_core(
@@ -100,6 +106,12 @@ async def speakers() -> Dict:
     return {"speakers": speakers}
 
 
+@sc_app.get("/macro/{macro_name}")
+async def run_commands(macro_name: str) -> Dict:
+    result = _process_macro(macro_name)
+    return {"result": result}
+
+
 @sc_app.get("/{speaker}/{action}")
 async def action_0(speaker: str, action: str) -> Dict:
     return command_core(speaker, action, use_local=USE_LOCAL)
@@ -142,6 +154,14 @@ def args_processor() -> None:
         help="Print the SoCo-CLI and SoCo versions, and exit",
     )
 
+    parser.add_argument(
+        "--macros",
+        "-m",
+        type=str,
+        default="macros.txt",
+        help="The file containing the local macros",
+    )
+
     args = parser.parse_args()
 
     if args.version:
@@ -152,17 +172,24 @@ def args_processor() -> None:
     if args.port is not None:
         PORT = args.port
 
+    global MACRO_FILE
+    MACRO_FILE = abspath(args.macros)
+
 
 def main() -> None:
     args_processor()
     print(PREFIX + "Starting " + INFO)
 
+    # Load local macros
+    global MACROS
+    _load_macros(MACROS, filename=MACRO_FILE)
+
     try:
         # Pre-load speaker cache
         print(PREFIX + "Finding speakers ... ", end="", flush=True)
         try:
             # This forces speaker discovery
-            # For some reason, using 'get_all_speakers()' provokes Uvicorn errors
+            # For some reason, using 'get_all_speakers()' generates Uvicorn errors
             get_speaker("")
             print(get_all_speaker_names())
         except:
@@ -178,5 +205,68 @@ def main() -> None:
         exit(1)
 
 
+def _process_macro(macro_name: str) -> str:
+    # Look up the macro
+    try:
+        macro = _lookup_macro(macro_name)
+    except KeyError:
+        print(PREFIX + "macro '{}' not found".format(macro_name))
+        return "Error: macro '{}' not found".format(macro_name)
+
+    # Substitute speaker names for IP addresses, for efficiency
+    sonos_command_line = "sonos " + _substitute_speaker_ips(macro)
+    print(PREFIX + "Executing: " + sonos_command_line)
+
+    # Execute the command
+    try:
+        output = check_output(sonos_command_line, shell=True)
+        return "Command line output: " + output.decode("utf-8")
+    except:
+        return "Error running command line: " + sonos_command_line
+
+
+def _lookup_macro(macro_name: str) -> str:
+    global MACROS
+    return MACROS[macro_name]
+
+
+def _substitute_speaker_ips(macro: str, use_local: bool = False) -> str:
+    """
+    Substitute speaker names for IP addresses, for efficiency.
+    Speaker names must be exact.
+    """
+    terms = shlex.split(macro)
+    new_macro_list = []
+    for term in terms:
+        device, error_msg = get_speaker(term, use_local_speaker_list=use_local)
+        if device is not None and device.player_name == term:
+            new_macro_list.append(device.ip_address)
+        else:
+            new_macro_list.append(term)
+    new_macro = (" ").join(new_macro_list)
+    return new_macro
+
+
+def _load_macros(macros: dict, filename: str) -> bool:
+    print(PREFIX + "Attempting to load macros from '{}'".format(filename))
+    try:
+        with open(filename, "r") as f:
+            line = f.readline()
+            while line != "":
+                if not line.startswith("#") and line != "\n":
+                    if line.count("=") != 1:
+                        print(PREFIX + "Malformed macro '{}'... ignored".format(line))
+                        print(line, end="")
+                    else:
+                        macro = line.split("=")
+                        macros[macro[0].strip()] = macro[1].strip()
+                line = f.readline()
+        print(PREFIX + "Loaded macros: {}".format(macros))
+        return True
+    except:
+        print(PREFIX + "Macro file not found")
+        return False
+
+
 if __name__ == "__main__":
     main()
